-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     memory_access_tb.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-11-19
-------------------------------------------------------------------------------
-- Brief    test bench of the fetche
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library work;
use work.riscv_pkg.all;

entity memory_access_tb is
end memory_access_tb;

architecture tb of memory_access_tb is

    constant XLEN   : positive := 32;

    signal store_data    : std_logic;
    signal alu_result    : std_logic_vector(XLEN-1 downto 0);
    signal rd_addr       : std_logic_vector(9 downto 0);
    signal rd_addr_o       : std_logic_vector(9 downto 0);
    signal clk           : std_logic;
    signal dmem_read     : std_logic_vector(XLEN-1 downto 0);
    signal dmem_addr      : std_logic_vector(9 downto 0);
    signal rstn : std_logic;
    
    signal dmem_read_o   : std_logic_vector(XLEN-1 downto 0);
    signal alu_result_o  : std_logic_vector(XLEN-1 downto 0);
    signal enable: std_logic;
    
    signal expected_result_alu : std_logic_vector(XLEN-1 downto 0);
    signal expected_result_dmem : std_logic_vector(XLEN-1 downto 0);
    signal expected_result_rd       : std_logic_vector(9 downto 0);
    signal expected_result_enable: std_logic;
    constant PERIOD   : time := 10 ns;

begin

  -- DUT
  dut: entity work.memory_access

    port map (
      i_clk   => clk,
      i_store_data => store_data,
      i_alu_result  => alu_result,
      i_rd_addr => rd_addr,
      i_rstn => rstn,
      
      o_alu_result => alu_result_o,
      o_dmem_read => dmem_read_o,
      o_rd_addr => rd_addr_o  ,
      
      --communication avec la DPM
      i_dmem_read =>dmem_read,
      o_dmem_addr=> dmem_addr, 
      o_dmem_en=> enable
       );

  drive_clk : process
  begin
    clk <= '0';
    wait for PERIOD/2;
    clk <= '1';
    wait for PERIOD/2;
  end process drive_clk;

  -- Main TB process
  p_main : process
  begin
    -- Vecteurs de tests

    report "BEGIN SIMULATION";
    --t=0 ns

    report "initialisation";
    rstn<='0';
    
  wait for PERIOD; --10 ns 
    rstn<='1' ;
    alu_result <= (0 downto 0 => '1', others => '0') ;
    rd_addr <= (0 downto 0 => '1', others => '0');
    store_data <= '0';
    dmem_read <= (others => '0');
  


  wait for PERIOD; --10 ns 
  
    store_data  <= '1';
    dmem_read <= (others => '1');
    
    expected_result_alu <= (0 downto 0 => '1', others => '0') ;
    expected_result_dmem <= (others => '0');
    expected_result_rd <= (0 downto 0 => '1', others => '0');   
    
    wait for PERIOD; --20 ns
    assert alu_result_o = expected_result_alu
      report "alu error" severity error;
    assert dmem_read_o = expected_result_dmem
      report "dmem read error" severity error;
    assert rd_addr = expected_result_rd
      report "rd_addr error" severity error;   

    expected_result_dmem <= (others => '1');
    expected_result_enable <= '1';
     
    
    wait for PERIOD; --30ns


    assert dmem_read_o = expected_result_dmem
      report "dmem read error" severity error;
    assert enable = expected_result_enable
      report "enable error" severity error;      

      
    report "SIMULATION DONE";
    wait;
  end process p_main;

end architecture tb;

